@@ -24,10 +24,18 @@
 
 #include <osmocom/ranap/ranap_common_ran.h>
 
+#include <osmocom/hnbgw/hnb_persistent.h>
 #include <osmocom/hnbgw/hnbgw_cn.h>
 #include <osmocom/hnbgw/context_map.h>
 #include <osmocom/hnbgw/kpi.h>
 
+const struct value_string hnbgw_rab_state_names[] = {
+	{ RAB_STATE_INACTIVE,	"INACTIVE" },
+	{ RAB_STATE_ACT_REQ,	"ACT_REQ" },
+	{ RAB_STATE_ACTIVE,	"ACTIVE" },
+	{ RAB_STATE_REL_REQ,	"REL_REQ" },
+	{}
+};
 
 /***********************************************************************
  * DOWNLINK messages
@@ -44,18 +52,16 @@ static void kpi_ranap_process_dl_iu_rel_cmd(struct hnbgw_context_map *map, const
 
 	/* When Iu is released, all RABs are released implicitly */
 	for (unsigned int i = 0; i < ARRAY_SIZE(map->rab_state); i++) {
-		unsigned int ctr_num;
 		switch (map->rab_state[i]) {
 		case RAB_STATE_ACTIVE:
 			if (cause->present == RANAP_Cause_PR_nAS ||
 			    cause->choice.nAS == RANAP_CauseNAS_normal_release) {
-				ctr_num = HNB_CTR_RANAP_PS_RAB_REL_IMPLICIT;
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_IMPLICIT :
+								HNB_CTR_RANAP_CS_RAB_REL_IMPLICIT);
 			} else {
-				ctr_num = HNB_CTR_RANAP_PS_RAB_REL_IMPLICIT_ABNORMAL;
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_IMPLICIT_ABNORMAL :
+								HNB_CTR_RANAP_CS_RAB_REL_IMPLICIT_ABNORMAL);
 			}
-			if (!map->is_ps)
-				ctr_num++;
-			HNBP_CTR_INC(hnbp, ctr_num);
 			break;
 		}
 	}
@@ -100,15 +106,20 @@ static void kpi_ranap_process_dl_rab_ass_req(struct hnbgw_context_map *map, rana
 			 * new, it is a setup */
 			switch (map->rab_state[rab_id]) {
 			case RAB_STATE_ACTIVE:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_REQ : HNB_CTR_RANAP_CS_RAB_MOD_REQ);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_REQ :
+								HNB_CTR_RANAP_CS_RAB_MOD_REQ);
 				break;
 			case RAB_STATE_INACTIVE:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_REQ : HNB_CTR_RANAP_CS_RAB_ACT_REQ);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_REQ :
+								HNB_CTR_RANAP_CS_RAB_ACT_REQ);
 				map->rab_state[rab_id] = RAB_STATE_ACT_REQ;
 				break;
 			default:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_REQ_UNEXP :
+								HNB_CTR_RANAP_CS_RAB_ACT_REQ_UNEXP);
 				LOG_MAP(map, DRANAP, LOGL_NOTICE,
-				       "Unexpected RAB Activation/Modification Req for RAB in state %u\n", map->rab_state[rab_id]);
+					"Unexpected RAB Activation/Modification Req for RAB in state %s\n",
+					hnbgw_rab_state_name(map->rab_state[rab_id]));
 				break;
 			}
 
@@ -123,7 +134,6 @@ static void kpi_ranap_process_dl_rab_ass_req(struct hnbgw_context_map *map, rana
 			RANAP_RAB_ReleaseItemIEs_t _rab_rel_item_ies = {};
 			RANAP_RAB_ReleaseItemIEs_t *rab_rel_item_ies = &_rab_rel_item_ies;
 			RANAP_RAB_ReleaseItem_t *rab_rel_item;
-			unsigned int ctr_num;
 			uint8_t rab_id;
 
 			if (!release_list_ie)
@@ -144,17 +154,19 @@ static void kpi_ranap_process_dl_rab_ass_req(struct hnbgw_context_map *map, rana
 			case RAB_STATE_ACTIVE:
 				if (rab_rel_item->cause.present == RANAP_Cause_PR_nAS &&
 				    rab_rel_item->cause.choice.nAS == RANAP_CauseNAS_normal_release) {
-					ctr_num = HNB_CTR_RANAP_PS_RAB_REL_REQ;
+					HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_REQ :
+									HNB_CTR_RANAP_CS_RAB_REL_REQ);
 				} else {
-					ctr_num = HNB_CTR_RANAP_PS_RAB_REL_REQ_ABNORMAL;
+					HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_REQ_ABNORMAL :
+									HNB_CTR_RANAP_CS_RAB_REL_REQ_ABNORMAL);
 				}
-				if (!map->is_ps)
-					ctr_num++;
-				HNBP_CTR_INC(hnbp, ctr_num);
 				break;
 			default:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_REQ_UNEXP :
+								HNB_CTR_RANAP_CS_RAB_REL_REQ_UNEXP);
 				LOG_MAP(map, DRANAP, LOGL_NOTICE,
-					"Unexpected RAB Release Req in state %u\n", map->rab_state[rab_id]);
+					"Unexpected RAB Release Req in state %s\n",
+					hnbgw_rab_state_name(map->rab_state[rab_id]));
 				break;
 			}
 			/* mark that RAB as release requested */
@@ -241,15 +253,20 @@ static void kpi_ranap_process_ul_rab_ass_resp(struct hnbgw_context_map *map, ran
 			/* differentiate modify / activate */
 			switch (map->rab_state[rab_id]) {
 			case RAB_STATE_ACT_REQ:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_CNF : HNB_CTR_RANAP_CS_RAB_ACT_CNF);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_CNF :
+								HNB_CTR_RANAP_CS_RAB_ACT_CNF);
 				map->rab_state[rab_id] = RAB_STATE_ACTIVE;
 				break;
 			case RAB_STATE_ACTIVE:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_CNF : HNB_CTR_RANAP_CS_RAB_MOD_CNF);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_CNF :
+								HNB_CTR_RANAP_CS_RAB_MOD_CNF);
 				break;
 			default:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_CNF_UNEXP :
+								HNB_CTR_RANAP_CS_RAB_ACT_CNF_UNEXP);
 				LOG_MAP(map, DRANAP, LOGL_NOTICE,
-				       "Unexpected RAB Activation/Modification Conf for RAB in state %u\n", map->rab_state[rab_id]);
+					"Unexpected RAB Activation/Modification Conf for RAB in state %s\n",
+					hnbgw_rab_state_name(map->rab_state[rab_id]));
 				break;
 			}
 
@@ -260,8 +277,6 @@ static void kpi_ranap_process_ul_rab_ass_resp(struct hnbgw_context_map *map, ran
 	if (ies->presenceMask & RAB_ASSIGNMENTRESPONSEIES_RANAP_RAB_RELEASEDLIST_PRESENT) {
 		RANAP_RAB_ReleasedList_t *r_list = &ies->raB_ReleasedList;
 		/* increment number of released RABs, we don't need to do that individually during iteration */
-		HNBP_CTR_ADD(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_CNF : HNB_CTR_RANAP_CS_RAB_REL_CNF,
-			     r_list->raB_ReleasedList_ies.list.count);
 		for (unsigned int i = 0; i < r_list->raB_ReleasedList_ies.list.count; i++) {
 			RANAP_IE_t *released_list_ie = r_list->raB_ReleasedList_ies.list.array[i];
 			RANAP_RAB_ReleasedItemIEs_t _rab_rel_item_ies = {};
@@ -285,10 +300,15 @@ static void kpi_ranap_process_ul_rab_ass_resp(struct hnbgw_context_map *map, ran
 
 			switch (map->rab_state[rab_id]) {
 			case RAB_STATE_REL_REQ:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_CNF :
+								HNB_CTR_RANAP_CS_RAB_REL_CNF);
 				break;
 			default:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_CNF_UNEXP :
+								HNB_CTR_RANAP_CS_RAB_REL_CNF_UNEXP);
 				LOG_MAP(map, DRANAP, LOGL_NOTICE,
-				       "Unexpected RAB Release Conf for RAB in state %u\n", map->rab_state[rab_id]);
+					"Unexpected RAB Release Conf for RAB in state %s\n",
+					hnbgw_rab_state_name(map->rab_state[rab_id]));
 				break;
 			}
 			/* mark that RAB as released */
@@ -327,16 +347,21 @@ static void kpi_ranap_process_ul_rab_ass_resp(struct hnbgw_context_map *map, ran
 			/* differentiate modify / activate */
 			switch (map->rab_state[rab_id]) {
 			case RAB_STATE_ACT_REQ:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_FAIL : HNB_CTR_RANAP_CS_RAB_ACT_FAIL);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_FAIL :
+								HNB_CTR_RANAP_CS_RAB_ACT_FAIL);
 				map->rab_state[rab_id] = RAB_STATE_INACTIVE;
 				break;
 			case RAB_STATE_ACTIVE:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_FAIL : HNB_CTR_RANAP_CS_RAB_MOD_FAIL);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_FAIL :
+								HNB_CTR_RANAP_CS_RAB_MOD_FAIL);
 				// FIXME: does it remain active after modification failure?
 				break;
 			default:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_FAIL_UNEXP :
+								HNB_CTR_RANAP_CS_RAB_ACT_FAIL_UNEXP);
 				LOG_MAP(map, DRANAP, LOGL_NOTICE,
-				       "Unexpected RAB Activation/Modification Failed for RAB in state %u\n", map->rab_state[rab_id]);
+					"Unexpected RAB Activation/Modification Failed for RAB in state %s\n",
+					hnbgw_rab_state_name(map->rab_state[rab_id]));
 				break;
 			}
 
@@ -347,8 +372,6 @@ static void kpi_ranap_process_ul_rab_ass_resp(struct hnbgw_context_map *map, ran
 	if (ies->presenceMask & RAB_ASSIGNMENTRESPONSEIES_RANAP_RAB_RELEASEFAILEDLIST_PRESENT) {
 		RANAP_RAB_ReleaseFailedList_t *rf_list = &ies->raB_ReleaseFailedList;
 		/* increment number of released RABs, we don't need to do that individually during iteration */
-		HNBP_CTR_ADD(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_FAIL : HNB_CTR_RANAP_CS_RAB_REL_FAIL,
-			     rf_list->raB_FailedList_ies.list.count);
 		for (unsigned int i = 0; i < rf_list->raB_FailedList_ies.list.count; i++) {
 			RANAP_IE_t *failed_list_ie = rf_list->raB_FailedList_ies.list.array[i];
 			RANAP_RAB_FailedItemIEs_t _rab_failed_item_ies = {};
@@ -373,16 +396,21 @@ static void kpi_ranap_process_ul_rab_ass_resp(struct hnbgw_context_map *map, ran
 			/* differentiate modify / activate */
 			switch (map->rab_state[rab_id]) {
 			case RAB_STATE_ACT_REQ:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_ACT_FAIL : HNB_CTR_RANAP_CS_RAB_ACT_FAIL);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_FAIL :
+								HNB_CTR_RANAP_CS_RAB_REL_FAIL);
 				map->rab_state[rab_id] = RAB_STATE_INACTIVE;
 				break;
 			case RAB_STATE_ACTIVE:
-				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_MOD_FAIL : HNB_CTR_RANAP_CS_RAB_MOD_FAIL);
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_FAIL :
+								HNB_CTR_RANAP_CS_RAB_REL_FAIL);
 				// FIXME: does it remain active after modification failure?
 				break;
 			default:
+				HNBP_CTR_INC(hnbp, map->is_ps ? HNB_CTR_RANAP_PS_RAB_REL_FAIL_UNEXP :
+								HNB_CTR_RANAP_CS_RAB_REL_FAIL_UNEXP);
 				LOG_MAP(map, DRANAP, LOGL_NOTICE,
-				       "Unexpected RAB Release Failed for RAB in state %u\n", map->rab_state[rab_id]);
+					"Unexpected RAB Release Failed for RAB in state %s\n",
+					hnbgw_rab_state_name(map->rab_state[rab_id]));
 				break;
 			}
 
